#include "StandardProblemIteration.h"
#include "AdaptInfo.h"
#include "ProblemStatBase.h"
#include "Global.h"
#include "OpenMP.h"

namespace AMDiS {

  int StandardProblemIteration::info_ = 10;

  void StandardProblemIteration::beginIteration(AdaptInfo *adaptInfo) 
  {
    FUNCNAME("StandardProblemIteration::beginIteration()");

    INFO(info_,4)("\n");
    INFO(info_,4)("begin of iteration number: %d\n", adaptInfo->getSpaceIteration() + 1);
    INFO(info_,4)("=============================\n");
  }

  Flag StandardProblemIteration::oneIteration(AdaptInfo *adaptInfo, Flag toDo) 
  {
    FUNCNAME("StandardProblemIteration::oneIteration()");

    Flag flag = buildAndAdapt(adaptInfo, toDo);

    if (toDo.isSet(SOLVE))
      problem_->solve(adaptInfo, false);

    if (toDo.isSet(SOLVE_RHS))
      problem_->solve(adaptInfo, true);

    if (toDo.isSet(ESTIMATE)) 
      problem_->estimate(adaptInfo);

    return flag;
  }

  void StandardProblemIteration::endIteration(AdaptInfo *adaptInfo) 
  {
    FUNCNAME("StandardProblemIteration::endIteration()");

    INFO(info_,4)("\n");
    INFO(info_,4)("end of iteration number: %d\n", adaptInfo->getSpaceIteration() + 1);
    INFO(info_,4)("=============================\n");
  }

  Flag StandardProblemIteration::buildAndAdapt(AdaptInfo *adaptInfo, Flag toDo)
  {
    FUNCNAME("StandardProblemIteration::buildAndAdapt()");

    Flag flag = 0, markFlag = 0;

    if (toDo.isSet(MARK)) {
      markFlag = problem_->markElements(adaptInfo);
    }  else {
      markFlag = 3;
    }

    if (toDo.isSet(BUILD)) {
      problem_->buildBeforeRefine(adaptInfo, markFlag);
    }

    // refine
    if (toDo.isSet(ADAPT) && markFlag.isSet(MESH_REFINED)) {
      flag = problem_->refineMesh(adaptInfo);
    }

    if (toDo.isSet(BUILD)) {
      problem_->buildBeforeCoarsen(adaptInfo, markFlag);
    }

    // coarsen
    if (toDo.isSet(ADAPT) && markFlag.isSet(MESH_COARSENED)) {
      flag |= problem_->coarsenMesh(adaptInfo);
    }

    if (toDo.isSet(BUILD)) {
      problem_->buildAfterCoarsen(adaptInfo, markFlag, true, true);
    }

    if (toDo.isSet(BUILD_RHS)) {
      problem_->buildAfterCoarsen(adaptInfo, markFlag, false, true);
    }

    return flag;
  }

  const std::string& StandardProblemIteration::getName()
  {
    return problem_->getName();
  }

  void StandardProblemIteration::serialize(std::ostream &out)
  {
    problem_->serialize(out);
  }

  void StandardProblemIteration::deserialize(std::istream &in)
  {
    problem_->deserialize(in);
  }
};
